# -*- coding: utf-8 -*-
from __future__ import print_function
import docutils
import os
import posixpath
import platform
import shutil
import unittest
from abjad.tools import abjadbooktools
from abjad.tools import stringtools
from sphinx.util import FilenameUniqDict


@unittest.skipIf(
    platform.python_implementation() != 'CPython',
    'Only for CPython.',
    )
class SphinxDocumentHandlerTests(unittest.TestCase):

    maxDiff = None

    class Namespace(object):
        pass

    def setUp(self):
        import abjad
        app = self.Namespace()
        config = self.Namespace()
        self.app = app
        self.app.config = config
        self.app.config.abjadbook_ignored_documents = ()
        self.app.builder = self.Namespace()
        self.app.builder.warn = print
        self.app.builder.current_docname = 'test'
        self.app.builder.status_iterator = lambda iterable, x, y, z: iter(iterable)
        self.app.builder.thumbnails = FilenameUniqDict()
        self.app.builder.outdir = os.path.dirname(os.path.abspath(__file__))
        self.app.builder.imagedir = '_images'
        self.app.builder.imgpath = posixpath.join('..', '_images')
        self.app.builder.srcdir = os.path.join(
            abjad.__path__[0],
            'docs',
            'source',
            )
        self.app.body = []
        self.images_directory = os.path.join(
            self.app.builder.outdir,
            self.app.builder.imagedir,
            )
        self.abjadbook_images_directory = os.path.join(
            self.images_directory,
            'abjadbook',
            )
        if os.path.exists(self.images_directory):
            shutil.rmtree(self.images_directory)

    def tearDown(self):
        if os.path.exists(self.images_directory):
            shutil.rmtree(self.images_directory)

    def test_01(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-stylesheet:

            show(Staff("c'4 d'4 e'4 f'4"))
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a href="../_images/abjadbook/lilypond-e16d48b9daaa5fc687733330eebf40e142583e58.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-e16d48b9daaa5fc687733330eebf40e142583e58.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 2
        for name in (
            'lilypond-e16d48b9daaa5fc687733330eebf40e142583e58.ly',
            'lilypond-e16d48b9daaa5fc687733330eebf40e142583e58.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_02(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-stylesheet:
            :no-trim:

            show(Staff("c'4 d'4 e'4 f'4"))
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a href="../_images/abjadbook/lilypond-9ff77da7bd81f083920be0b350b205fd6b6767a1.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-9ff77da7bd81f083920be0b350b205fd6b6767a1.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 2
        for name in (
            'lilypond-9ff77da7bd81f083920be0b350b205fd6b6767a1.ly',
            'lilypond-9ff77da7bd81f083920be0b350b205fd6b6767a1.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_03(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-resize:
            :no-stylesheet:
            :no-trim:

            show(Staff("c'4 d'4 e'4 f'4"))
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a href="../_images/abjadbook/lilypond-28a967840c6a267316825bff73ccd5418c06ed04.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-28a967840c6a267316825bff73ccd5418c06ed04.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 2
        for name in (
            'lilypond-28a967840c6a267316825bff73ccd5418c06ed04.ly',
            'lilypond-28a967840c6a267316825bff73ccd5418c06ed04.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_04(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-resize:
            :no-stylesheet:
            :no-trim:
            :with-thumbnail:

            show(Staff("c'4 d'4 e'4 f'4"))
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        assert len(self.app.builder.thumbnails) == 1
        assert '../_images/abjadbook/lilypond-28a967840c6a267316825bff73ccd5418c06ed04.png' in self.app.builder.thumbnails
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a data-lightbox="group-lilypond-28a967840c6a267316825bff73ccd5418c06ed04.ly" href="../_images/abjadbook/lilypond-28a967840c6a267316825bff73ccd5418c06ed04.png" title="" data-title="" class="abjadbook thumbnail">
                <img src="../_images/abjadbook/lilypond-28a967840c6a267316825bff73ccd5418c06ed04-thumbnail.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 3
        for name in (
            'lilypond-28a967840c6a267316825bff73ccd5418c06ed04.ly',
            'lilypond-28a967840c6a267316825bff73ccd5418c06ed04.png',
            'lilypond-28a967840c6a267316825bff73ccd5418c06ed04-thumbnail.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_05(self):
        source = r'''
        ..  abjad::
            :hide:
            :stylesheet: non-proportional.ly
            :no-trim:

            show(Staff("c'4 d'4 e'4 f'4"))
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        abjadbooktools.SphinxDocumentHandler.on_builder_inited(self.app)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a href="../_images/abjadbook/lilypond-be904c180d556273c3bd4c6e53c4636e5c7aaf4f.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-be904c180d556273c3bd4c6e53c4636e5c7aaf4f.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 6
        for name in (
            'default.ly',
            'external-settings-file-1.ly',
            'external-settings-file-2.ly',
            'lilypond-be904c180d556273c3bd4c6e53c4636e5c7aaf4f.ly',
            'lilypond-be904c180d556273c3bd4c6e53c4636e5c7aaf4f.png',
            'non-proportional.ly',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_06(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-stylesheet:

            staff = Staff("c'1 d'1 e'1 f'1 g'1")
            for note in staff[:-1]:
                attach(indicatortools.PageBreak(), note)

            show(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page1.png" alt=""/>
            </a>
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page2.png" alt=""/>
            </a>
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page3.png" alt=""/>
            </a>
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page4.png" alt=""/>
            </a>
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page5.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 6
        for name in (
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page1.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page2.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page3.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page4.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page5.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_07(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-stylesheet:
            :pages: 2-4

            staff = Staff("c'1 d'1 e'1 f'1 g'1")
            for note in staff[:-1]:
                attach(indicatortools.PageBreak(), note)

            show(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page2.png" alt=""/>
            </a>
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page3.png" alt=""/>
            </a>
            <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="abjadbook">
                <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page4.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 6
        for name in (
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page1.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page2.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page3.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page4.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page5.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_08(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-stylesheet:
            :pages: 2-4
            :with-columns: 2

            staff = Staff("c'1 d'1 e'1 f'1 g'1")
            for note in staff[:-1]:
                attach(indicatortools.PageBreak(), note)

            show(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <div class="table-row">
                <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="table-cell">
                    <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page2.png" alt=""/>
                </a>
                <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="table-cell">
                    <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page3.png" alt=""/>
                </a>
            </div>
            <div class="table-row">
                <a href="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly" title="" class="table-cell">
                    <img src="../_images/abjadbook/lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page4.png" alt=""/>
                </a>
            </div>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 6
        for name in (
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5.ly',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page1.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page2.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page3.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page4.png',
            'lilypond-d8a292fabe1535f5ab562e33bc3c97216675f8a5-page5.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_09(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-stylesheet:
            :no-trim:
            :pages: 2-4
            :with-columns: 2

            staff = Staff("c'1 d'1 e'1 f'1 g'1")
            for note in staff[:-1]:
                attach(indicatortools.PageBreak(), note)

            show(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <div class="table-row">
                <a href="../_images/abjadbook/lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315.ly" title="" class="table-cell">
                    <img src="../_images/abjadbook/lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page2.png" alt=""/>
                </a>
                <a href="../_images/abjadbook/lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315.ly" title="" class="table-cell">
                    <img src="../_images/abjadbook/lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page3.png" alt=""/>
                </a>
            </div>
            <div class="table-row">
                <a href="../_images/abjadbook/lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315.ly" title="" class="table-cell">
                    <img src="../_images/abjadbook/lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page4.png" alt=""/>
                </a>
            </div>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 6
        for name in (
            'lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315.ly',
            'lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page1.png',
            'lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page2.png',
            'lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page3.png',
            'lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page4.png',
            'lilypond-6900efb6ce21bef42b89ef019e69f08e070d7315-page5.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_10(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-resize:
            :no-stylesheet:
            :with-thumbnail:

            staff = Staff("c'1 d'1 e'1 f'1 g'1")
            for note in staff[:-1]:
                attach(indicatortools.PageBreak(), note)

            show(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1.png" title="" data-title="" class="abjadbook thumbnail">
                <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1-thumbnail.png" alt=""/>
            </a>
            <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2.png" title="" data-title="" class="abjadbook thumbnail">
                <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2-thumbnail.png" alt=""/>
            </a>
            <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3.png" title="" data-title="" class="abjadbook thumbnail">
                <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3-thumbnail.png" alt=""/>
            </a>
            <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4.png" title="" data-title="" class="abjadbook thumbnail">
                <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4-thumbnail.png" alt=""/>
            </a>
            <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5.png" title="" data-title="" class="abjadbook thumbnail">
                <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5-thumbnail.png" alt=""/>
            </a>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 11
        for name in (
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5-thumbnail.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)

    def test_11(self):
        source = r'''
        ..  abjad::
            :hide:
            :no-resize:
            :no-stylesheet:
            :with-columns: 2
            :with-thumbnail:

            staff = Staff("c'1 d'1 e'1 f'1 g'1")
            for note in staff[:-1]:
                attach(indicatortools.PageBreak(), note)

            show(staff)
        '''
        source = stringtools.normalize(source)
        handler = abjadbooktools.SphinxDocumentHandler()
        document = handler.parse_rst(source)
        handler.on_doctree_read(self.app, document)
        node = document[0]
        try:
            abjadbooktools.SphinxDocumentHandler.visit_abjad_output_block_html(
                self.app, node)
        except docutils.nodes.SkipNode:
            pass
        handler.on_build_finished(self.app, None)
        actual = '\n'.join(self.app.body)
        expected = stringtools.normalize(r'''
            <div class="table-row">
                <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1.png" title="" data-title="" class="table-cell thumbnail">
                    <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1-thumbnail.png" alt=""/>
                </a>
                <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2.png" title="" data-title="" class="table-cell thumbnail">
                    <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2-thumbnail.png" alt=""/>
                </a>
            </div>
            <div class="table-row">
                <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3.png" title="" data-title="" class="table-cell thumbnail">
                    <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3-thumbnail.png" alt=""/>
                </a>
                <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4.png" title="" data-title="" class="table-cell thumbnail">
                    <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4-thumbnail.png" alt=""/>
                </a>
            </div>
            <div class="table-row">
                <a data-lightbox="group-lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly" href="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5.png" title="" data-title="" class="table-cell thumbnail">
                    <img src="../_images/abjadbook/lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5-thumbnail.png" alt=""/>
                </a>
            </div>
            ''')
        self.assertEqual(actual, expected)
        assert len(os.listdir(self.abjadbook_images_directory)) == 11
        for name in (
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e.ly',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page1-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page2-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page3-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page4-thumbnail.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5.png',
            'lilypond-e95e4239a7b2d051083422fe9ff795547273601e-page5-thumbnail.png',
            ):
            path = os.path.join(self.images_directory, 'abjadbook', name)
            assert os.path.exists(path)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval_numbered_interval_number_01():

    assert pitchtools.NumberedInterval(-14).numbered_interval_number == -14
    assert pitchtools.NumberedInterval(14).numbered_interval_number == 14
    assert pitchtools.NumberedInterval(-2).numbered_interval_number == -2
    assert pitchtools.NumberedInterval(2).numbered_interval_number == 2



# -*- coding: utf-8 -*-
from abjad import *


def test_lilypondfiletools_make_floating_time_signature_lilypond_file_01():

    staff = Staff("c'8 d'8 e'8 f'8")
    lilypond_file = \
        lilypondfiletools.make_floating_time_signature_lilypond_file(staff)
    context_block = lilypond_file.layout_block.items[1]

    assert format(context_block) == stringtools.normalize(
        r'''
        \context {
            \name TimeSignatureContext
            \type Engraver_group
            \consists Axis_group_engraver
            \consists Time_signature_engraver
            \override TimeSignature.X-extent = #'(0 . 0)
            \override TimeSignature.X-offset = #ly:self-alignment-interface::x-aligned-on-self
            \override TimeSignature.Y-extent = #'(0 . 0)
            \override TimeSignature.break-align-symbol = ##f
            \override TimeSignature.break-visibility = #end-of-line-invisible
            \override TimeSignature.font-size = #1
            \override TimeSignature.self-alignment-X = #center
            \override VerticalAxisGroup.default-staff-staff-spacing = #'((basic-distance . 0) (minimum-distance . 10) (padding . 6) (stretchability . 0))
        }
        '''
        )



# -*- coding: utf-8 -*-
import math


# TODO: replace string-valued even='allowed' with constant-valued keyword
def partition_integer_into_halves(n, bigger=Left, even='allowed'):
    r'''Partitions `n` into halves.
    
    Writes positive integer `n` as the pair ``t = (left, right)`` such that
    ``n == left + right``.

    ..   container:: example

        When `n` is odd the greater part of ``t`` corresponds to the value of
        `bigger`:

        ::

            >>> mathtools.partition_integer_into_halves(7, bigger=Left)
            (4, 3)

        ::

            >>> mathtools.partition_integer_into_halves(7, bigger=Right)
            (3, 4)

    ..  container:: example

        Likewise when `n` is even and ``even = 'disallowed'``:

        ::

            >>> mathtools.partition_integer_into_halves(
            ...     8,
            ...     bigger=Left,
            ...     even='disallowed',
            ...     )
            (5, 3)

        ::

            >>> mathtools.partition_integer_into_halves(
            ...     8,
            ...     bigger=Right,
            ...     even='disallowed',
            ...     )
            (3, 5)

    ..  container:: example

        But when `n` is even and ``even = 'allowed'`` then ``left == right``
        and `bigger` is ignored:

        ::

            >>> mathtools.partition_integer_into_halves(8)
            (4, 4)

        ::

            >>> mathtools.partition_integer_into_halves(8, bigger=Left)
            (4, 4)

        ::

            >>> mathtools.partition_integer_into_halves(8, bigger=Right)
            (4, 4)

    ..  container:: example

        When `n` is ``0`` returns ``(0, 0)``:

        ::

            >>> mathtools.partition_integer_into_halves(0)
            (0, 0)

    When `n` is ``0`` and ``even = 'disallowed'`` raises partition error.

    Returns pair of positive integers.
    '''
    if not isinstance(n, int):
        raise TypeError
    if n < 0:
        raise ValueError
    if n == 0:
        if even == 'disallowed':
            raise Exception
        return (0, 0)
    smaller_half = int(math.floor(n / 2))
    bigger_half = n - smaller_half
    if (smaller_half == bigger_half) and (even != 'allowed'):
        smaller_half -= 1
        bigger_half += 1
    if bigger == Left:
        return (bigger_half, smaller_half)
    else:
        return (smaller_half, bigger_half)



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Accidental___eq___01():
    r'''Accidentals compare equal when they carry the same string.
    '''
    assert pitchtools.Accidental('ff') == pitchtools.Accidental('ff')
    assert pitchtools.Accidental('tqf') == pitchtools.Accidental('tqf')
    assert pitchtools.Accidental('f') == pitchtools.Accidental('f')
    assert pitchtools.Accidental('qf') == pitchtools.Accidental('qf')
    assert pitchtools.Accidental('qs') == pitchtools.Accidental('qs')
    assert pitchtools.Accidental('s') == pitchtools.Accidental('s')
    assert pitchtools.Accidental('tqs') == pitchtools.Accidental('tqs')
    assert pitchtools.Accidental('ss') == pitchtools.Accidental('ss')


def test_pitchtools_Accidental___eq___02():
    r'''Accidentals compare equal when they carry no string.
    '''
    assert pitchtools.Accidental() == pitchtools.Accidental()
    assert pitchtools.Accidental('') == pitchtools.Accidental('')
    assert pitchtools.Accidental() == pitchtools.Accidental('')


def test_pitchtools_Accidental___eq___03():
    r'''Accidentals do not compare equal to a naked string.
    '''
    assert not pitchtools.Accidental('s') == 's'



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_scoretools_Container___copy___01():
    r'''Containers copy simultaneity flag.
    '''

    container_1 = Container([Voice("c'8 d'8"), Voice("c''8 b'8")])
    container_1.is_simultaneous = True
    container_2 = copy.copy(container_1)


    assert format(container_1) == stringtools.normalize(
        r'''
        <<
            \new Voice {
                c'8
                d'8
            }
            \new Voice {
                c''8
                b'8
            }
        >>
        '''
        )

    assert format(container_2) == stringtools.normalize(
        r'''
        <<
        >>
        '''
        )

    assert container_1 is not container_2



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ScaleDegree__initialize_by_symbolic_string_01():

    scale_degree = tonalanalysistools.ScaleDegree('I')
    assert scale_degree == tonalanalysistools.ScaleDegree(1)

    scale_degree = tonalanalysistools.ScaleDegree('i')
    assert scale_degree == tonalanalysistools.ScaleDegree(1)

    scale_degree = tonalanalysistools.ScaleDegree('bII')
    assert scale_degree == tonalanalysistools.ScaleDegree('flat', 2)

    scale_degree = tonalanalysistools.ScaleDegree('bii')
    assert scale_degree == tonalanalysistools.ScaleDegree('flat', 2)


def test_tonalanalysistools_ScaleDegree__initialize_by_symbolic_string_02():

    scale_degree = tonalanalysistools.ScaleDegree('1')
    assert scale_degree == tonalanalysistools.ScaleDegree(1)

    scale_degree = tonalanalysistools.ScaleDegree('b2')
    assert scale_degree == tonalanalysistools.ScaleDegree('flat', 2)

    scale_degree = tonalanalysistools.ScaleDegree('#4')
    assert scale_degree == tonalanalysistools.ScaleDegree('sharp', 4)



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import selectiontools


def make_multimeasure_rests(durations):
    '''Make multi-measure rests from `durations`:

    ::

        >>> scoretools.make_multimeasure_rests([(4, 4), (7, 4)])
        Selection([MultimeasureRest('R1'), MultimeasureRest('R1..')])

    Returns list.
    '''
    from abjad.tools import scoretools

    multimeasure_rests = []

    for duration in durations:
        written_duration = durationtools.Duration(duration)
        if not written_duration.is_assignable:
            message = 'multimeasure rest durations must be assignable.'
            raise AssignabilityError(message)
        multimeasure_rest = scoretools.MultimeasureRest(written_duration)
        multimeasure_rests.append(multimeasure_rest)

    result = selectiontools.Selection(multimeasure_rests)
    return result



# -*- coding: utf-8 -*-
from abjad.tools.abctools import AbjadValueObject


class BowPressure(AbjadValueObject):
    r'''Bow pressure indicator.

    ..  container:: example

        **Example 1.** Overpressure indicator:

        ..  container:: example

            ::

                >>> bow_pressure = indicatortools.BowPressure('overpressure')
                >>> print(format(bow_pressure))
                indicatortools.BowPressure(
                    pressure='overpressure',
                    )

    ..  container:: example

        **Example 2.** Underpressure indicator:

        ..  container:: example

            ::

                >>> bow_pressure = indicatortools.BowPressure('underpressure')
                >>> print(format(bow_pressure))
                indicatortools.BowPressure(
                    pressure='underpressure',
                    )

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_pressure',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        pressure=None,
        ):
        self._default_scope = None
        self._pressure = pressure

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of bow pressure indicator.

        ..  container:: example

            ::

                >>> bow_pressure = indicatortools.BowPressure('overpressure')
                >>> bow_pressure.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def pressure(self):
        r'''Gets pressure of indicator.

        ..  container:: example

            **Example 1.** Overpressure:

            ::

                >>> bow_pressure = indicatortools.BowPressure('overpressure')
                >>> bow_pressure.pressure
                'overpressure'

        ..  container:: example

            **Example 2.** Underpressure:

            ::

                >>> bow_pressure = indicatortools.BowPressure('underpressure')
                >>> bow_pressure.pressure
                'underpressure'

        Returns string.
        '''
        return self._pressure



# -*- coding: utf-8 -*-
from abjad import *


def test_stringtools_is_space_delimited_lowercase_01():

    assert stringtools.is_space_delimited_lowercase('foo bar blah')
    assert stringtools.is_space_delimited_lowercase('foo bar')
    assert stringtools.is_space_delimited_lowercase('foo')
    assert stringtools.is_space_delimited_lowercase('f')


def test_stringtools_is_space_delimited_lowercase_02():

    assert stringtools.is_space_delimited_lowercase('')


def test_stringtools_is_space_delimited_lowercase_03():

    assert not stringtools.is_space_delimited_lowercase('foo_bar')
    assert not stringtools.is_space_delimited_lowercase('foo_bar blah')
    assert not stringtools.is_space_delimited_lowercase('Foo_Bar')
    assert not stringtools.is_space_delimited_lowercase('FOO_BAR')



# -*- coding: utf-8 -*-
from __future__ import print_function
import doctest
import importlib
import os
import sys
from abjad.tools import stringtools
from abjad.tools import systemtools
from abjad.tools.commandlinetools.CommandlineScript import CommandlineScript
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class DoctestScript(CommandlineScript):
    r'''Runs doctests on all Python files in current directory recursively.

    ..  shell::

        ajv doctest --help

    '''

    ### CLASS VARIABLES ###

    alias = 'doctest'
    short_description = 'Run doctests on all modules in current path.'

    _module_names_for_globs = (
        'abjad',
        'experimental',
        )

    ### PRIVATE METHODS ###

    def _get_file_paths(self, input_paths):
        ignored_directory_names = (
            '__pycache__',
            '.git',
            '.svn',
            'test',
            'docs',
            )
        file_paths = []
        for input_path in input_paths:
            if not os.path.exists(input_path):
                print('No such path {!r}'.format(input_path))
                sys.exit(1)
            elif os.path.isfile(input_path):
                file_paths.append(input_path)
            for current_root, directories, files in os.walk(input_path):
                for directory in directories[:]:
                    if directory in ignored_directory_names:
                        directories.remove(directory)
                    elif not os.path.exists(os.path.join(
                        current_root, directory, '__init__.py')):
                        directories.remove(directory)
                for file_name in files[:]:
                    if not file_name.endswith(('.py', '.pyx')):
                        continue
                    file_path = os.path.join(current_root, file_name)
                    file_paths.append(file_path)
        return file_paths

    def _get_namespace(self, abjad_only=False):
        globs = {}
        if abjad_only:
            globs['abjad'] = importlib.import_module('abjad')
            globs['f'] = getattr(globs['abjad'], 'f')
        else:
            for module_name in self._module_names_for_globs:
                try:
                    module = importlib.import_module(module_name)
                    globs.update(module.__dict__)
                except:
                    pass
        try:
            ide_module = importlib.import_module('ide')
            globs['ide'] = ide_module
        except ImportError:
            pass
        globs['print_function'] = print_function
        config_parser = self._config_parser
        try:
            imports = config_parser.get(self.alias, 'imports')
        except:
            imports = ''
        imports = stringtools.normalize(imports).split('\n')
        for line in imports:
            exec(line, globs, globs)
        globs
        return globs

    def _get_optionflags(self, arguments):
        optionflags = (
            doctest.NORMALIZE_WHITESPACE |
            doctest.ELLIPSIS
            )
        if arguments and arguments.diff:
            optionflags = optionflags | doctest.REPORT_NDIFF
        if arguments and arguments.x:
            optionflags = optionflags | doctest.REPORT_ONLY_FIRST_FAILURE
        return optionflags

    def _process_args(
        self,
        arguments=None,
        file_paths=None,
        print_to_terminal=True,
        ):
        assert not (arguments and file_paths)
        result = []
        globs = self._get_namespace(abjad_only=arguments.abjad_only)
        optionflags = self._get_optionflags(arguments)
        total_failures = 0
        total_modules = 0
        total_tests = 0
        failed_file_paths = []
        error_messages = []
        file_paths = file_paths or self._get_file_paths(arguments.path)
        try:
            for file_path in sorted(file_paths):
                total_modules += 1
                relative_path = os.path.relpath(file_path)
                string_buffer = StringIO()
                with systemtools.RedirectedStreams(stdout=string_buffer):
                    failure_count, test_count = doctest.testfile(
                        file_path,
                        module_relative=False,
                        globs=globs,
                        optionflags=optionflags,
                        )
                doctest_output = string_buffer.getvalue()
                if failure_count:
                    failed_file_paths.append(os.path.relpath(file_path))
                    error_messages.append(doctest_output)
                    if print_to_terminal:
                        result_code = ''.join((
                            self._colors['RED'],
                            'FAILED',
                            self._colors['END'],
                            ))
                        print(relative_path, result_code)
                    else:
                        result_code = 'FAILED'
                        string = '{} {}'.format(relative_path, result_code)
                        result.append(string)
                    if arguments and arguments.x:
                        break
                else:
                    if print_to_terminal:
                        result_code = ''.join((
                            self._colors['BLUE'],
                            'OK',
                            self._colors['END'],
                            ))
                        print(relative_path, result_code)
                    else:
                        result_code = 'OK'
                        string = '{} {}'.format(relative_path, result_code)
                        result.append(string)
                total_failures += failure_count
                total_tests += test_count
        except KeyboardInterrupt:
            print('Interrupted. Halting tests.')
        self._report(
            error_messages=error_messages,
            failed_file_paths=failed_file_paths,
            print_to_terminal=print_to_terminal,
            result=result,
            total_tests=total_tests,
            total_failures=total_failures,
            total_modules=total_modules,
            )

    def _report(
        self,
        error_messages=None,
        failed_file_paths=None,
        print_to_terminal=False,
        result=None,
        total_tests=None,
        total_failures=None,
        total_modules=None,
        ):
        if failed_file_paths:
            if print_to_terminal:
                print()
            else:
                result.append('')
            for error_message in error_messages:
                if print_to_terminal:
                    print(error_message)
                else:
                    result.append(error_message)
        for file_path in failed_file_paths:
            string = 'FAILED: {}'.format(file_path)
            if print_to_terminal:
                print(string)
            else:
                result.append(string)
        if print_to_terminal:
            print()
        else:
            result.append('')
        test_identifier = stringtools.pluralize('test', total_tests)
        module_identifier = stringtools.pluralize('module', total_modules)
        string = (
            '{total_successes} passed, {total_failures} failed out of '
            '{total_tests} {test_identifier} '
            'in {total_modules} {module_identifier}.'
            )
        string = string.format(
            module_identifier=module_identifier,
            test_identifier=test_identifier,
            total_failures=total_failures,
            total_modules=total_modules,
            total_successes=total_tests - total_failures,
            total_tests=total_tests,
            )
        if print_to_terminal:
            print(string)
            if failed_file_paths:
                sys.exit(1)
            else:
                sys.exit(0)
        else:
            result.append(string)
            return result

    def _setup_argument_parser(self, parser):
        parser.add_argument(
            'path',
            default=[os.getcwd()],
            help='directory tree to be recursed over',
            nargs='*',
            )
        parser.add_argument(
            '--abjad-only',
            action='store_true',
            help='load abjad but not * from abjad.',
            )
        parser.add_argument(
            '--diff',
            action='store_true',
            help='print diff-like output on failed tests.',
            )
        parser.add_argument(
            '-x',
            action='store_true',
            help='stop after first failure.',
            )



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Glissando_01():

    staff = Staff([Note(n, (1, 8)) for n in range(8)])
    glissando = spannertools.Glissando()
    attach(glissando, staff[:4])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 \glissando
            cs'8 \glissando
            d'8 \glissando
            ef'8
            e'8
            f'8
            fs'8
            g'8
        }
        '''
        )


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_IntervalSegment_slope_01():
    mcis = pitchtools.IntervalSegment([1])
    assert mcis.slope == 1


def test_pitchtools_IntervalSegment_slope_02():
    mcis = pitchtools.IntervalSegment([-2, 1])
    assert mcis.slope == Fraction(-1, 2)


def test_pitchtools_IntervalSegment_slope_03():
    mcis = pitchtools.IntervalSegment([1, 2, -3, 0, 2, -1, -1.5, 0.5])
    assert mcis.slope == 0



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_schemetools_SchemeColor___setattr___01():
    r'''Scheme colors are immutable.
    '''

    scheme_color = schemetools.SchemeColor('ForestGreen')
    assert pytest.raises(AttributeError, "scheme_color.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_rhythmtreetools_RhythmTreeNode_proper_parentage_01():

    leaf = rhythmtreetools.RhythmTreeLeaf()
    subsubcontainer = rhythmtreetools.RhythmTreeContainer()
    subcontainer = rhythmtreetools.RhythmTreeContainer()
    container = rhythmtreetools.RhythmTreeContainer()

    container.append(subcontainer)
    subcontainer.append(subsubcontainer)
    subsubcontainer.append(leaf)

    assert leaf.proper_parentage == (subsubcontainer, subcontainer, container)
    assert subsubcontainer.proper_parentage == (subcontainer, container)
    assert subcontainer.proper_parentage == (container,)
    assert container.proper_parentage == ()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_Leaf__split_by_duration_01():
    r'''Split note into assignable notes.
    Don't fracture spanners. Don't tie split notes.
    '''

    staff = Staff("c'8 [ d'8 e'8 ]")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    halves = staff[1]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=False,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'32
            d'16.
            e'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_02():
    r'''Split note into assignable notes.
    Fracture spanners. But don't tie split notes.
    '''

    staff = Staff("c'8 [ d'8 e'8 ]")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    halves = staff[1]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'32 ]
            d'16. [
            e'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_03():
    r'''Split note into assignable notes.
    Don't fracture spanners. But do tie split notes.
    '''

    staff = Staff("c'8 [ d'8 e'8 ]")

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'8
            e'8 ]
        }
        '''
        )

    halves = staff[1]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=False,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'32 ~
            d'16.
            e'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_04():
    r'''Split note into assignable notes.
    Fracture spanners and tie split notes.
    '''

    staff = Staff("c'8 [ d'8 e'8 ]")

    halves = staff[1]._split_by_duration(
        Duration(1, 32),
        fracture_spanners=True,
        tie_split_notes=True,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            d'32 ~ ]
            d'16. [
            e'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_05():
    r'''Split note into tuplet monads.
    Don't fracture spanners. Don't tie split notes.
    '''

    staff = Staff("c'8 [ d'8 e'8 ]")

    halves = staff[1]._split_by_duration(
        Duration(1, 24),
        tie_split_notes=False,
        )

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            c'8 [
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                d'16
            }
            \tweak edge-height #'(0.7 . 0)
            \times 2/3 {
                d'8
            }
            e'8 ]
        }
        '''
        )

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_06():
    r'''Notehead-assignable duration produces two notes.
    This test comes from a container-crossing spanner bug.
    '''

    voice = Voice(r"c'8 \times 2/3 { d'8 e'8 f'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, leaves)

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            \times 2/3 {
                d'8
                e'8
                f'8 ]
            }
        }
        '''
        )

    halves = leaves[1]._split_by_duration(
        Duration(1, 24),
        tie_split_notes=False,
        )

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            c'8 [
            \times 2/3 {
                d'16
                d'16
                e'8
                f'8 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_scoretools_Leaf__split_by_duration_07():
    r'''Split duration equal to zero produces no change.
    '''

    note = Note("c'4")

    halves = note._split_by_duration(Duration(0))
    left, right = halves

    assert len(halves) == 2
    assert len(left) == 0
    assert len(right) == 1
    assert isinstance(right[0], Note)
    assert right[0].written_duration == Duration(1, 4)


def test_scoretools_Leaf__split_by_duration_08():
    r'''Leaf duration less than split duration produces no change.
    '''

    note = Note("c'4")

    halves = note._split_by_duration(Duration(3, 4))
    left, right = halves

    assert len(halves) == 2
    assert len(left) == 1
    assert isinstance(left[0], Note)
    assert left[0].written_duration == Duration(1, 4)
    assert len(right) == 0


def test_scoretools_Leaf__split_by_duration_09():
    r'''Split returns two lists of zero or more leaves.
    '''

    note = Note("c'4")

    halves = note._split_by_duration(
        Duration(1, 8),
        tie_split_notes=False,
        )

    assert isinstance(halves, tuple)
    assert len(halves) == 2
    assert len(halves[0]) == 1
    assert len(halves[1]) == 1
    assert halves[0][0] is note
    assert halves[1][0] is not note
    assert isinstance(halves[0][0], Note)
    assert isinstance(halves[1][0], Note)
    assert halves[0][0].written_duration == Duration(1, 8)
    assert halves[1][0].written_duration == Duration(1, 8)
    assert len(inspect_(halves[0][0]).get_logical_tie()) == 1
    assert len(inspect_(halves[1][0]).get_logical_tie()) == 1


def test_scoretools_Leaf__split_by_duration_10():
    r'''Split returns two lists of zero or more.
    '''

    note = Note("c'4")
    halves = note._split_by_duration(Duration(1, 16))

    assert isinstance(halves, tuple)
    assert len(halves) == 2
    assert len(halves[0]) == 1
    assert len(halves[1]) == 1
    assert isinstance(halves[0][0], Note)
    assert isinstance(halves[1][0], Note)
    assert halves[0][0].written_duration == Duration(1, 16)
    assert halves[1][0].written_duration == Duration(3, 16)


def test_scoretools_Leaf__split_by_duration_11():
    r'''Nonassignable split duration with power-of-two denominator
    produces two lists.
    Left list contains two notes tied together.
    Right list contains only one note.
    '''

    note = Note("c'4")

    halves = note._split_by_duration(
        Duration(5, 32),
        tie_split_notes=False,
        )

    assert isinstance(halves, tuple)
    assert len(halves) == 2
    assert len(halves[0]) == 2
    assert len(halves[1]) == 1
    assert isinstance(halves[0][0], Note)
    assert isinstance(halves[0][1], Note)
    assert isinstance(halves[1][0], Note)
    assert halves[0][0].written_duration == Duration(4, 32)
    assert halves[0][1].written_duration == Duration(1, 32)
    assert halves[1][0].written_duration == Duration(3, 32)
    assert len(inspect_(halves[0][0]).get_logical_tie()) == 2
    assert len(inspect_(halves[0][1]).get_logical_tie()) == 2
    assert len(inspect_(halves[1][0]).get_logical_tie()) == 1


def test_scoretools_Leaf__split_by_duration_12():
    r'''Lone spanned Leaf results in two spanned leaves.
    '''

    staff = Staff([Note("c'4")])
    tie = spannertools.Tie()
    attach(tie, staff[:])
    halves = staff[0]._split_by_duration(Duration(1, 8))

    assert len(staff) == 2
    for leaf in staff[:]:
        assert inspect_(leaf).get_spanners() == set([tie])
        prototype = (spannertools.Tie,)
        assert inspect_(leaf).get_spanner(prototype) is tie

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_13():
    r'''Spanners are unaffected by leaf split.
    '''

    staff = Staff("c'8 c'8 c'8 c'8")
    beam = Beam()
    attach(beam, staff[:])

    halves = staff[0]._split_by_duration(
        Duration(1, 16),
        tie_split_notes=False,
        )

    assert len(staff) == 5
    for l in staff:
        assert inspect_(l).get_spanners() == set([beam])
        assert l._get_spanner(Beam) is beam

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_14():
    r'''Split returns three leaves, two are tied.
    Spanner is shared by all 3 leaves.
    '''

    staff = Staff([Note("c'4")])
    tie = spannertools.Tie()
    attach(tie, staff[:])
    halves = staff[0]._split_by_duration(Duration(5, 32))

    assert len(halves) == 2
    assert len(halves[0]) == 2
    assert len(halves[1]) == 1
    for l in staff:
        assert inspect_(l).get_spanners() == set([tie])
        assert inspect_(l).get_spanner(spannertools.Tie) is tie

    assert inspect_(staff).is_well_formed()


def test_scoretools_Leaf__split_by_duration_15():
    r'''After grace notes are removed from first leaf in bipartition.
    '''

    note = Note("c'4")
    grace = scoretools.GraceContainer([Note(0, (1, 32))], kind='after')
    attach(grace, note)
    halves = note._split_by_duration(Duration(1, 8))

    assert not hasattr(halves[0][0], 'after_grace')
    after_grace = inspect_(halves[1][0]).get_grace_container()
    assert len(after_grace) == 1


def test_scoretools_Leaf__split_by_duration_16():
    r'''After grace notes are removed from first tied leaves in bipartition.
    '''

    note = Note("c'4")
    grace = scoretools.GraceContainer([Note(0, (1, 32))], kind='after')
    attach(grace, note)
    halves = note._split_by_duration(Duration(5, 32))

    assert len(halves) == 2
    assert getattr(halves[0][0], 'after_grace', None) is None
    assert getattr(halves[0][1], 'after_grace', None) is None
    assert len(halves[1]) == 1
    after_grace = inspect_(halves[1][0]).get_grace_container()
    assert len(after_grace) == 1


def test_scoretools_Leaf__split_by_duration_17():
    r'''Grace notes are removed from second leaf in bipartition.
    '''

    note = Note("c'4")
    grace = scoretools.GraceContainer([Note(0, (1, 32))])
    attach(grace, note)
    halves = note._split_by_duration(Duration(1, 16))

    assert len(halves[0]) == 1
    assert len(halves[1]) == 1
    grace_container = inspect_(halves[0][0]).get_grace_container()
    assert len(grace_container) == 1
    assert not hasattr(halves[1][0], 'grace') is None



# -*- coding: utf-8 -*-
from abjad.tools.durationtools.Duration import Duration


class Multiplier(Duration):
    '''Multiplier.

    ..  container:: example

        **Example 1.** Initializes from integer numerator:

        ::

            >>> Multiplier(3)
            Multiplier(3, 1)

    ..  container:: example

        **Example 2.** Initializes from integer numerator and denominator:

        ::

            >>> Multiplier(3, 16)
            Multiplier(3, 16)

    ..  container:: example

        **Example 3.** Initializes from integer-equivalent numeric numerator:

        ::

            >>> Multiplier(3.0)
            Multiplier(3, 1)

    ..  container:: example

        **Example 4.** Initializes from integer-equivalent numeric numerator
        and denominator:

        ::

            >>> Multiplier(3.0, 16)
            Multiplier(3, 16)

    ..  container:: example

        **Example 5.** Initializes from integer-equivalent singleton:

        ::

            >>> Multiplier((3,))
            Multiplier(3, 1)

    ..  container:: example

        **Example 6.** Initializes from integer-equivalent pair:

        ::

            >>> Multiplier((3, 16))
            Multiplier(3, 16)

    ..  container:: example

        **Example 7.** Initializes from other duration:

        ::

            >>> Multiplier(Duration(3, 16))
            Multiplier(3, 16)

    ..  container:: example

        **Example 8.** Intializes from fraction:

        ::

            >>> Multiplier(Fraction(3, 16))
            Multiplier(3, 16)

    ..  container:: example

        **Example 9.** Initializes from solidus string:

        ::

            >>> Multiplier('3/16')
            Multiplier(3, 16)

    ..  container:: example

        **Example 10.** Initializes from nonreduced fraction:

        ::

            >>> Multiplier(mathtools.NonreducedFraction(6, 32))
            Multiplier(3, 16)

    ..  container:: example

        **Example 11.** Multipliers inherit from built-in fraction:

        ::

            >>> isinstance(Multiplier(3, 16), Fraction)
            True

    ..  container:: example

        **Example 12.** Multipliers are numeric:

        ::

            >>> import numbers

        ::

            >>> isinstance(Multiplier(3, 16), numbers.Number)
            True

    ..  container:: example

        **Example 13.** Attaching a multiplier to a score component multiplies
        that component's duration:

        ::

            >>> note = Note("c'1")
            >>> print(format(note))
            c'1

        ::

            >>> multiplier = Multiplier(3, 8)
            >>> attach(multiplier, note)
            >>> print(format(note))
            c'1 * 3/8

    '''

    ### CLASS VARIABLES ###

    __slots__ = ()

    ### SPECIAL METHODS ###

    def __mul__(self, *arguments):
        r'''Multiplier times duration gives duration.

        Returns duration.
        '''
        if len(arguments) == 1 and type(arguments[0]) is Duration:
            return Duration(Duration.__mul__(self, *arguments))
        else:
            return Duration.__mul__(self, *arguments)

    ### PUBLIC PROPERTIES ###

    @property
    def is_proper_tuplet_multiplier(self):
        '''Is true when mutliplier is greater than ``1/2`` and less than ``2``.
        Otherwise false:

        ::

            >>> Multiplier(3, 2).is_proper_tuplet_multiplier
            True

        Returns true or false.
        '''
        return type(self)(1, 2) < self < type(self)(2)


# -*- coding: utf-8 -*-
from abjad import *
from abjad.demos import part


def test_demos_part_create_pitch_contour_reservoir_01():

    reservoir = part.create_pitch_contour_reservoir()

    assert isinstance(reservoir, dict)
    for key, value in reservoir.items():
        assert isinstance(key, str)
        assert isinstance(value, tuple)
        assert len(value)



# -*- coding: utf-8 -*-


# TODO: move to Sequence
def cumulative_sums_pairwise(argument):
    r'''Gets pairwise cumulative sums of `argument` from zero.

    ..  container:: example

        ::

            >>> mathtools.cumulative_sums_pairwise([1, 2, 3, 4, 5, 6])
            [(0, 1), (1, 3), (3, 6), (6, 10), (10, 15), (15, 21)]

    Returns pairs in new object of `argument` type.
    '''
    from abjad.tools import mathtools
    from abjad.tools import sequencetools
    sums = mathtools.cumulative_sums(argument)
    pairs = sequencetools.Sequence(sums).nwise()
    return type(argument)([tuple(_) for _ in pairs])



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Voice___len___01():
    r'''Voice length returns the number of elements in voice.
    '''

    voice = Voice()
    assert len(voice) == 0


def test_scoretools_Voice___len___02():
    r'''Voice length returns the number of elements in voice.
    '''

    voice = Voice("c'8 d'8 e'8 f'8")
    assert len(voice) == 4



# -*- coding: utf-8 -*-
import functools
from abjad.tools import durationtools
from abjad.tools import markuptools
from abjad.tools.abctools import AbjadValueObject


@functools.total_ordering
class BowContactPoint(AbjadValueObject):
    r'''Bow contact point.

    ..  container:: example

        **Example 1.** Contact point exactly halfway from frog to tip of bow:

        ::

            >>> point = indicatortools.BowContactPoint((1, 2))
            >>> print(format(point))
            indicatortools.BowContactPoint(
                contact_point=durationtools.Multiplier(1, 2),
                )

    ..  container:: example

        **Example 2.** Contact point 3/5 of the way from frog to tip of bow:

        ::

            >>> point = indicatortools.BowContactPoint((3, 5))
            >>> print(format(point))
            indicatortools.BowContactPoint(
                contact_point=durationtools.Multiplier(3, 5),
                )

    Contact points are measured from frog to tip as a fraction between ``0`` 
    and ``1``.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contact_point',
        '_default_scope'
        )

    ### INITIALIZER ###

    def __init__(
        self,
        contact_point=None,
        ):
        if contact_point is not None:
            contact_point = durationtools.Multiplier(contact_point)
            assert 0 <= contact_point <= 1
        self._contact_point = contact_point
        self._default_scope = None

    ### SPECIAL METHODS ###

    def __lt__(self, argument):
        r'''Is true if `argument` is a bow contact point and this bow contact point
        is less than `argument`.

        ..  container:: example

            ::

                >>> point_1 = indicatortools.BowContactPoint((1, 2))
                >>> point_2 = indicatortools.BowContactPoint((1, 2))
                >>> point_3 = indicatortools.BowContactPoint((2, 3))

            ::

                >>> point_1 < point_1
                False
                >>> point_1 < point_2
                False
                >>> point_1 < point_3
                True

            ::

                >>> point_2 < point_1
                False
                >>> point_2 < point_2
                False
                >>> point_2 < point_3
                True

            ::

                >>> point_3 < point_1
                False
                >>> point_3 < point_2
                False
                >>> point_3 < point_3
                False

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.contact_point < argument.contact_point
        raise TypeError('unorderable types')

    ### PUBLIC PROPERTIES ###

    @property
    def contact_point(self):
        r'''Gets contact point of bow contact point.

        ..  container:: example

            **Example 1.** One quarter of the way from frog to point:

            ::

                >>> point = indicatortools.BowContactPoint((1, 4))
                >>> point.contact_point
                Multiplier(1, 4)

        ..  container:: example

            **Example 2.** Three fifths of the way from frog to point:

            ::

                >>> point = indicatortools.BowContactPoint((3, 5))
                >>> point.contact_point
                Multiplier(3, 5)

        Returns multiplier.
        '''
        return self._contact_point

    @property
    def markup(self):
        r'''Gets markup of bow contact point.

        ..  container:: example

            **Example 1.** One quarter of the way from frog to point:

            ::

                >>> indicator = indicatortools.BowContactPoint((1, 4))
                >>> print(format(indicator.markup, 'lilypond'))
                \markup {
                    \center-align
                        \vcenter
                            \fraction
                                1
                                4
                    }
                >>> show(indicator.markup) # doctest: +SKIP

        ..  container:: example

            **Example 2.** Three fifths of the way from frog to point:

            ::

                >>> indicator = indicatortools.BowContactPoint((3, 5))
                >>> print(format(indicator.markup, 'lilypond'))
                \markup {
                    \center-align
                        \vcenter
                            \fraction
                                3
                                5
                    }
                >>> show(indicator.markup) # doctest: +SKIP

        Returns markup.
        '''
        markup = markuptools.Markup.fraction(
            self.contact_point.numerator,
            self.contact_point.denominator,
            )
        markup = markup.vcenter()
        markup = markup.center_align()
        return markup



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class Arpeggio(AbjadValueObject):
    r'''Arpeggio.

    ..  container:: example

        **Example 1.** Without direction arrow:

        ::

            >>> chord = Chord("<c' e' g' c''>4")
            >>> arpeggio = indicatortools.Arpeggio()
            >>> attach(arpeggio, chord)
            >>> show(chord) # doctest: +SKIP

        ..  doctest::

            >>> print(format(chord))
            <c' e' g' c''>4 \arpeggio

    ..  container:: example

        **Example 2.** With direction arrow:

        ::

            >>> chord = Chord("<c' e' g' c''>4")
            >>> arpeggio = indicatortools.Arpeggio(direction=Down)
            >>> attach(arpeggio, chord)
            >>> show(chord) # doctest: +SKIP

        ..  doctest::

            >>> print(format(chord))
            \arpeggioArrowDown
            <c' e' g' c''>4 \arpeggio

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_direction',
        )

    ### INITIALIZER ###

    def __init__(self, direction=None):
        self._default_scope = None
        if direction is not None:
            assert direction in (Up, Down, Center)
        self._direction = direction

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return r'\arpeggio'

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        lilypond_format_bundle.right.articulations.append(r'\arpeggio')
        if self.direction in (Up, Down):
            if self.direction == Up:
                command = r'\arpeggioArrowUp'
            else:
                command = r'\arpeggioArrowDown'
            lilypond_format_bundle.before.commands.append(command)
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of arpeggio.

        ..  container:: example

            ::

                >>> arpeggio = indicatortools.Arpeggio()
                >>> arpeggio.default_scope is None
                True

        Returns none.
        '''
        return self._default_scope

    @property
    def direction(self):
        r'''Gets direction of arpeggio.

        ..  container:: example

            **Example 1.** Without direction arrow:

            ::

                >>> arpeggio = indicatortools.Arpeggio()
                >>> arpeggio.direction is None
                True

        ..  container:: example

            **Example 2.** With direction arrow:

            ::

                >>> arpeggio = indicatortools.Arpeggio(direction=Down)
                >>> arpeggio.direction
                Down

        Returns ordinal constant or none.
        '''
        return self._direction



# -*- coding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import schemetools
from abjad.tools.topleveltools import override


def make_colored_text_spanner_with_nibs():
    r'''Makes colored text spanner with nibs.

    ..  container:: example

        ::

            >>> staff = Staff("c'8 d'8 e'8 f'8")
            >>> spanner = spannertools.make_colored_text_spanner_with_nibs()
            >>> attach(spanner, staff[:])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> f(staff)
            \new Staff {
                \override TextSpanner.bound-details.left-broken.text = ##f
                \override TextSpanner.bound-details.left.text = \markup {
                    \draw-line
                        #'(0 . -1)
                    }
                \override TextSpanner.bound-details.right-broken.text = ##f
                \override TextSpanner.bound-details.right.text = \markup {
                    \draw-line
                        #'(0 . -1)
                    }
                \override TextSpanner.color = #red
                \override TextSpanner.dash-fraction = #1
                \override TextSpanner.staff-padding = #2
                \override TextSpanner.thickness = #1.5
                c'8 \startTextSpan
                d'8
                e'8
                f'8 \stopTextSpan
                \revert TextSpanner.bound-details
                \revert TextSpanner.color
                \revert TextSpanner.dash-fraction
                \revert TextSpanner.staff-padding
                \revert TextSpanner.thickness
            }

    Renders 1.5-unit thick solid red spanner.

    Draws nibs at beginning and end of spanner.

    Does not draw nibs at line breaks.

    Returns bracket spanner.
    '''
    from abjad.tools import spannertools

    spanner = spannertools.TextSpanner()
    pair = schemetools.SchemePair(0, -1)
    markup_command = markuptools.MarkupCommand('draw-line', pair)
    markup = markuptools.Markup(markup_command)
    override(spanner).text_spanner.bound_details__left__text = markup
    override(spanner).text_spanner.bound_details__left_broken__text = False
    markup = markuptools.Markup(markup)
    override(spanner).text_spanner.bound_details__right__text = markup
    override(spanner).text_spanner.bound_details__right_broken__text = False
    override(spanner).text_spanner.color = 'red'
    override(spanner).text_spanner.dash_fraction = 1
    override(spanner).text_spanner.staff_padding = 2
    override(spanner).text_spanner.thickness = 1.5

    return spanner



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Container_grob_override_01():
    r'''Noncontext containers bracket grob overrides at opening and closing.
    '''

    container = Container("c'8 d'8 e'8 f'8")
    override(container).glissando.thickness = 3

    assert format(container) == stringtools.normalize(
        r'''
        {
            \override Glissando.thickness = #3
            c'8
            d'8
            e'8
            f'8
            \revert Glissando.thickness
        }
        '''
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_Spanner_extend_01():
    r'''Extend spanner to the right.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    beam = Beam()
    attach(beam, voice[1][:])

    beam._extend(voice[2][:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                d'8
            }
            {
                e'8 [
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }
        '''
        )

    assert inspect_(voice).is_well_formed()


def test_spannertools_Spanner_extend_02():
    r'''Extend spanner to the right.
    '''

    voice = Voice("{ c'8 d'8 } { e'8 f'8 } { g'8 a'8 }")
    leaves = select(voice).by_leaf()
    beam = Beam()
    attach(beam, voice[1][:])
    beam._extend(leaves[-2:])

    assert format(voice) == stringtools.normalize(
        r'''
        \new Voice {
            {
                c'8
                d'8
            }
            {
                e'8 [
                f'8
            }
            {
                g'8
                a'8 ]
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
import argparse
import doctest
import re
import unittest
try:
    import pathlib
except ImportError:
    import pathlib2 as pathlib
try:
    from StringIO import StringIO
except ImportError:
    from io import StringIO


class TestCase(unittest.TestCase):

    ### CLASS VARIABLES ###

    ansi_escape = re.compile(r'\x1b[^m]*m')

    ### PUBLIC METHODS ###

    def compare_captured_output(self, expected):
        from abjad.tools import stringtools
        actual = self.ansi_escape.sub('', self.string_io.getvalue())
        actual = stringtools.normalize(actual)
        expected = stringtools.normalize(expected)
        self.compare_strings(expected, actual)

    def compare_file_contents(self, path, expected_contents):
        from abjad.tools import stringtools
        expected_contents = stringtools.normalize(expected_contents)
        with open(str(path), 'r') as file_pointer:
            contents = stringtools.normalize(file_pointer.read())
        self.compare_strings(expected_contents, contents)

    def compare_lilypond_contents(self, ly_path, expected_contents):
        from abjad.tools import stringtools
        expected_contents = stringtools.normalize(expected_contents)
        with open(str(ly_path), 'r') as file_pointer:
            contents = file_pointer.read()
        if ly_path.suffix == '.ly':
            contents = contents.splitlines()
            while 'version' not in contents[0]:
                contents.pop(0)
            contents.pop(0)
            contents = '\n'.join(contents)
        contents = stringtools.normalize(contents)
        self.compare_strings(expected_contents, contents)

    def compare_path_contents(self, path_to_search, expected_files):
        actual_files = sorted(
            str(path.relative_to(self.test_path))
            for path in path_to_search.glob('**/*.*')
            if '__pycache__' not in path.parts and
            path.suffix != '.pyc'
            )
        self.compare_strings(
            '\n'.join(str(_) for _ in actual_files),
            '\n'.join(str(_) for _ in expected_files),
            )

    def compare_strings(self, expected, actual):
        actual = self.normalize(self.ansi_escape.sub('', actual))
        expected = self.normalize(self.ansi_escape.sub('', expected))
        example = argparse.Namespace()
        example.want = expected
        output_checker = doctest.OutputChecker()
        flags = (
            doctest.NORMALIZE_WHITESPACE |
            doctest.ELLIPSIS |
            doctest.REPORT_NDIFF
            )
        success = output_checker.check_output(expected, actual, flags)
        if not success:
            diff = output_checker.output_difference(example, actual, flags)
            raise Exception(diff)

    def normalize(self, string):
        from abjad.tools import stringtools
        return stringtools.normalize(string)

    def reset_string_io(self):
        self.string_io.close()
        self.string_io = StringIO()

    def setUp(self):
        self.string_io = StringIO()

    def tearDown(self):
        self.string_io.close()

    ### PUBLIC PROPERTIES ###

    @property
    def test_path(self):
        return pathlib.Path(__file__).parent



# -*- coding: utf-8 -*-
from __future__ import print_function
import bisect
import collections
from abjad.tools import durationtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class MeterFittingSession(AbjadValueObject):
    r'''A meter-fitting session.

    Used internally by Meter.fit_meters_to_expr().
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_cached_offset_counters',
        '_kernel_denominator',
        '_kernels',
        '_longest_kernel',
        '_maximum_run_length',
        '_meters',
        '_offset_counter',
        '_ordered_offsets',
        )

    KernelScore = collections.namedtuple(
        'KernelScore',
        ('kernel', 'score'),
        )

    ### INITIALIZER ###

    def __init__(
        self,
        kernel_denominator=32,
        maximum_run_length=None,
        meters=None,
        offset_counter=None,
        ):
        from abjad.tools import metertools
        self._cached_offset_counters = {}
        if maximum_run_length is not None:
            maximum_run_length = int(maximum_run_length)
            assert 0 < maximum_run_length
        self._maximum_run_length = maximum_run_length
        if offset_counter:
            self._offset_counter = \
                metertools.MetricAccentKernel.count_offsets(
                    offset_counter)
        else:
            self._offset_counter = {}
        self._ordered_offsets = tuple(sorted(self.offset_counter))
        meters = meters or ()
        self._meters = tuple(metertools.Meter(_) for _ in meters)
        self._kernel_denominator = durationtools.Duration(kernel_denominator)
        self._kernels = {}
        for meter in self._meters:
            kernel = meter.generate_offset_kernel_to_denominator(
                self._kernel_denominator)
            self._kernels[kernel] = meter
        if self.kernels:
            self._longest_kernel = sorted(
                self._kernels,
                key=lambda x: x.duration,
                )[-1]
        else:
            self._longest_kernel = None

    ### SPECIAL METHODS ###

    def __call__(self):
        r'''Fits meters.

        Returns meter list.
        '''
        from abjad.tools import metertools
        selected_kernels = []
        current_offset = durationtools.Offset(0)
        while current_offset < self.ordered_offsets[-1]:
            kernel_scores = []
            kernels = self._get_kernels(selected_kernels)
            offset_counter = self._get_offset_counter_at(current_offset)
            if not offset_counter:
                winning_kernel = self.longest_kernel
                if selected_kernels:
                    winning_kernel = selected_kernels[-1]
            else:
                for kernel in kernels:
                    if self.maximum_run_length and \
                        1 < len(kernels) and \
                        self.maximum_run_length <= len(selected_kernels):
                        last_n_kernels = \
                            selected_kernels[-self.maximum_run_length:]
                        if len(set(last_n_kernels)) == 1:
                            if kernel == last_n_kernels[-1]:
                                continue
                    initial_score = kernel(offset_counter)
                    lookahead_score = self._get_lookahead_score(
                        current_offset,
                        kernel,
                        kernels,
                        )
                    score = initial_score + lookahead_score
                    kernel_score = self.KernelScore(
                        kernel=kernel,
                        score=score,
                        )
                    kernel_scores.append(kernel_score)
                kernel_scores.sort(key=lambda kernel_score: kernel_score.score)
                winning_kernel = kernel_scores[-1].kernel
            selected_kernels.append(winning_kernel)
            current_offset += winning_kernel.duration
        selected_meters = (self.kernels[_] for _ in selected_kernels)
        selected_meters = metertools.MeterList(selected_meters)
        return selected_meters

    ### PRIVATE METHODS ###

    def _get_kernels(self, selected_kernels):
        return tuple(self.kernels)

    def _get_lookahead_score(self, current_offset, kernel, kernels):
        lookahead_scores = []
        lookahead_offset = current_offset + kernel.duration
        lookahead_offset_counter = self._get_offset_counter_at(
            lookahead_offset)
        for lookahead_kernel in kernels:
            lookahead_scores.append(
                lookahead_kernel(lookahead_offset_counter)
                )
        lookahead_score = sum(lookahead_scores)  # / len(lookahead_scores)
        return lookahead_score

    def _get_offset_counter_at(self, start_offset):
        if start_offset in self.cached_offset_counters:
            return self.cached_offset_counters[start_offset]
        offset_counter = {}
        stop_offset = start_offset + self.longest_kernel.duration
        index = bisect.bisect_left(self.ordered_offsets, start_offset)
        if index == len(self.ordered_offsets):
            return offset_counter
        offset = self.ordered_offsets[index]
        while offset <= stop_offset:
            count = self.offset_counter[offset]
            offset_counter[offset - start_offset] = count
            index += 1
            if index == len(self.ordered_offsets):
                break
            offset = self.ordered_offsets[index]
        self.cached_offset_counters[start_offset] = offset_counter
        return offset_counter

    ### PUBLIC PROPERTIES ###

    @property
    def cached_offset_counters(self):
        r'''Gets cached offset counters

        Returns dictionary.
        '''
        return self._cached_offset_counters

    @property
    def kernel_denominator(self):
        r'''Gets kernel denominator.

        Returns duration.
        '''
        return self._kernel_denominator

    @property
    def kernels(self):
        r'''Gets kernels-to-meter dictionary.

        Returns dictionary.
        '''
        return self._kernels

    @property
    def longest_kernel(self):
        r'''Gets longest kernel.

        Returns kernel.
        '''
        return self._longest_kernel

    @property
    def maximum_run_length(self):
        r'''Gets maximum meter repetitions.

        Returns integer or none.
        '''
        return self._maximum_run_length

    @property
    def meters(self):
        r'''Gets meters.

        Returns meters.
        '''
        return self._meters

    @property
    def offset_counter(self):
        r'''Gets offset counter.

        Returns offset counter.
        '''
        return self._offset_counter

    @property
    def ordered_offsets(self):
        r'''Gets ordered offsets.

        Returns offsets.
        '''
        return self._ordered_offsets



# -*- coding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools import pitchtools
from abjad.tools.pitchtools import PitchClassSet


class RootedChordClass(PitchClassSet):
    '''A rooted chord class.

    ..  container:: example

        G major triad in root position:

        ::

            >>> tonalanalysistools.RootedChordClass('g', 'major')
            GMajorTriadInRootPosition

    ..  container:: example

        G dominant seventh in root position:

        ::

            >>> chord_class = tonalanalysistools.RootedChordClass('g', 'dominant', 7)

    Note that notions like a G dominant seventh represent an entire class of
    chords because there are many different spacings and registrations of a G
    dominant seventh.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_bass',
        '_chord_quality',
        '_root',
        )

    _cardinality_to_extent = {
        3: 5,
        4: 7,
        5: 9,
        6: 11,
        7: 13,
        }

    _extent_to_cardinality = {
        5: 3,
        7: 4,
        9: 5,
        11: 6,
        13: 7,
        }

    _extent_to_extent_name = {
        5: 'triad',
        7: 'seventh',
        9: 'ninth',
        11: 'eleventh',
        13: 'thirteenth',
        }

    ### INITIALIZER ###

    def __init__(self, root=None, *arguments):
        from abjad.tools import tonalanalysistools
        root = root or 'c'
        root = pitchtools.NamedPitchClass(root)
        chord_quality = tonalanalysistools.RootlessChordClass(*arguments)
        npcs = []
        for hdi in chord_quality:
            mdi = pitchtools.NamedInterval(hdi)
            npc = root + mdi
            npcs.append(npc)
        bass = npcs[0]
        PitchClassSet.__init__(
            self,
            items=npcs,
            item_class=pitchtools.NamedPitchClass,
            )
        self._root = root
        self._chord_quality = chord_quality
        self._bass = bass

    ### SPECIAL METHODS ###

    def __eq__(self, arg):
        r'''Is true when `arg` is a rooted chord-class with root, chord quality
        and inversion equal to those of this rooted chord-class. Otherwise
        false.

        Returns true or false.
        '''
        if isinstance(arg, type(self)):
            if self.root == arg.root:
                if self.chord_quality == arg.chord_quality:
                    if self.inversion == arg.inversion:
                        return True
        return False

    def __hash__(self):
        r'''Hashes rooted chord-class.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(RootedChordClass, self).__hash__()

    def __ne__(self, arg):
        r'''Is true when rooted chord-class does not equal `arg`. Otherwise
        false.

        Returns true or false.
        '''
        return not self == arg

    def __repr__(self):
        r'''Gets interpreter representation of rooted chord-class.

        Returns string.
        '''
        root = str(self.root).title()
        quality = self.chord_quality._title_case_name
        return root + quality

    ### PUBLIC METHODS ###

    @staticmethod
    def cardinality_to_extent(cardinality):
        r'''Change `cardinality` to extent.

        ..  container:: example

            Tertian chord with four pitch classes qualifies as a seventh chord:

            ::

                >>> tonalanalysistools.RootedChordClass.cardinality_to_extent(4)
                7

        Returns integer.
        '''
        return RootedChordClass._cardinality_to_extent[cardinality]

    @staticmethod
    def extent_to_cardinality(extent):
        r'''Change `extent` to cardinality.

        ..  container:: example

            Tertian chord with extent of seven
            comprises four pitch-clases:

            ::

                >>> tonalanalysistools.RootedChordClass.extent_to_cardinality(7)
                4

        Returns integer.
        '''
        return RootedChordClass._extent_to_cardinality[extent]

    @staticmethod
    def extent_to_extent_name(extent):
        r'''Change `extent` to extent name.

        ..  container:: example

            Extent of seven is a seventh:

            ::

                >>> tonalanalysistools.RootedChordClass.extent_to_extent_name(7)
                'seventh'

        Returns string.
        '''
        return RootedChordClass._extent_to_extent_name[extent]

    def transpose(self):
        r'''Transpose rooted chord-class.

        Not yet implemented.

        Will return new rooted chord-class.
        '''
        raise NotImplementedError

    ### PRIVATE PROPERTIES ###

    @property
    def _markup_root(self):
        if self.chord_quality._quality_string in (
            'major', 'augmented', 'dominant'):
            root = str(self.root).upper()
        else:
            root = str(self.root).lower()
        if len(root) == 2:
            adjustment = r'\hspace #-0.5 \raise #1 \fontsize #-3'
            if root[-1].lower() == 's':
                root = root[0] + r'{} \sharp'.format(adjustment)
            elif root[-1].lower() == 'f':
                root = root[0] + r'{} \flat'.format(adjustment)
            else:
                message = 'unknown note name: {}'
                message = message.format(root)
                raise ValueError(message)
        return root

    @property
    def _markup_symbol(self):
        circle = r'\draw-circle #0.35 #0 ##f'
        if self.chord_quality._quality_string == 'augmented':
            return '+'
        elif self.chord_quality._quality_string == 'diminished':
            return circle
        elif self.chord_quality._quality_string == 'half diminished':
            line = r"\draw-line #'(1 . 1)"
            markup = r'\concat {{ {} \hspace #-0.85 \raise #-0.5 {} }}'.format(
                circle, line)
            return markup
        elif self.chord_quality._quality_string == 'major' and \
            5 < self.extent.number:
            return 'M'
        elif self.chord_quality._quality_string == 'minor' and \
            5 < self.extent.number:
            return 'm'
        else:
            return ''

    ### PUBLIC PROPERTIES ###

    @property
    def bass(self):
        r'''Bass of rooted chord-class.

        ::

            >>> chord_class.bass
            NamedPitchClass('g')

        Returns named pitch-class.
        '''
        return self._bass

    @property
    def cardinality(self):
        r'''Cardinality of rooted chord-class.

        ::

            >>> chord_class.cardinality
            4

        Returns nonnegative integer.
        '''
        return len(self)

    @property
    def chord_quality(self):
        r'''Chord quality of rooted chord-class.

        ::

            >>> chord_class.chord_quality
            DominantSeventhInRootPosition('P1', '+M3', '+P5', '+m7')

        Returns chord quality.
        '''
        return self._chord_quality

    @property
    def extent(self):
        r'''Extent of rooted chord-class.

        ::

            >>> chord_class.extent
            ChordExtent(number=7)

        Returns chord extent.
        '''
        from abjad.tools import tonalanalysistools
        extent = self.cardinality_to_extent(self.cardinality)
        return tonalanalysistools.ChordExtent(extent)

    @property
    def figured_bass(self):
        r'''Figured bass of rooted chord-class.

        ::

            >>> chord_class.figured_bass
            '7'

        Returns string.
        '''
        extent, inversion = self.extent, self.inversion
        if extent.number == 5:
            if inversion == 0:
                return ''
            elif inversion == 1:
                return '6/3'
            elif inversion == 2:
                return '6/4'
        elif extent.number == 7:
            if inversion == 0:
                return '7'
            elif inversion == 1:
                return '6/5'
            elif inversion == 2:
                return '4/3'
            elif inversion == 3:
                return '4/2'
        elif extent.number == 9:
            if inversion == 0:
                return ''
            elif inversion == 1:
                raise NotImplementedError
            elif inversion == 2:
                raise NotImplementedError
            elif inversion == 3:
                raise NotImplementedError
            elif inversion == 4:
                raise NotImplementedError

    @property
    def inversion(self):
        r'''Inversion of rooted chord-class.

        ::

            >>> chord_class.inversion
            0

        Returns nonnegative integer.
        '''
        return self._chord_quality.inversion

    @property
    def markup(self):
        r'''Markup of rooted chord-class.

        ::

            >>> show(chord_class.markup) # doctest: +SKIP

        Returns markup.
        '''
        markup = [self._markup_root, self._markup_symbol, self.figured_bass]
        markup = ''.join(markup)
        markup = r'\fontsize #1 {} \hspace #-0.5'.format(self._markup_root)
        symbol = self._markup_symbol
        if symbol:
            markup += r' \hspace #0.5 \raise #1 \fontsize #-3 {}'.format(
                symbol)
            if 'circle' in symbol:
                if 'sharp' in self._markup_root:
                    markup += r' \hspace #0'
                else:
                    markup += r' \hspace #-1.2'
        inversion = self.figured_bass
        if inversion:
            inv = r" \raise #1 \fontsize #-3 \override #'(baseline-skip . 1.5)"
            inv += r' \column {{ {} }}'.format(' '.join(inversion.split('/')))
            markup += inv
        return markuptools.Markup(markup, Down)

    @property
    def quality_pair(self):
        r'''Quality pair of rooted chord-class.

        ::

            >>> chord_class.quality_pair
            ('dominant', 'seventh')

        Returns pair.
        '''
        chord_quality = self.chord_quality
        return chord_quality.quality_string, chord_quality.extent_name

    @property
    def root(self):
        r'''Root of rooted chord-class.

        ::

            >>> chord_class.root
            NamedPitchClass('g')

        Returns
        '''
        return self._root

    @property
    def root_string(self):
        r'''Root string of rooted chord-class.

        ::

            >>> chord_class.root_string
            'G'

        Returns string.
        '''
        capitalized_qualities = ('major', 'dominant', 'augmented')
        symbolic_name = self.root.pitch_class_label
        letter, accidental = symbolic_name[0], symbolic_name[1:]
        if self.chord_quality.quality_string in capitalized_qualities:
            letter = letter.upper()
        else:
            letter = letter.lower()
        return letter + accidental



# -*- coding: utf-8 -*-
from abjad import *


def test_timespantools_Timespan_curtails_timespan_01():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, -5)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_02():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 0)
    assert timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_03():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 5)
    assert timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_04():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 15)
    assert timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_05():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(-10, 25)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_06():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 10)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_07():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 15)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_08():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 10)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_09():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 15)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_10():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(0, 25)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_11():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(5, 25)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_12():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(15, 25)
    assert not timespan_1.curtails_timespan(timespan_2)

def test_timespantools_Timespan_curtails_timespan_13():
    timespan_1 = timespantools.Timespan(0, 15)
    timespan_2 = timespantools.Timespan(20, 25)
    assert not timespan_1.curtails_timespan(timespan_2)



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_PitchClassSegment___slots___01():

    pitch_class_segment = pitchtools.PitchClassSegment([
        'gs', 'a', 'as', 'c', 'cs'])
    assert pytest.raises(AttributeError, "pitch_class_segment.foo = 'bar'")


def test_pitchtools_PitchClassSegment___slots___02():

    pitch_class_tokens = [-2, -1.5, 6, 7, -1.5, 7]
    pitch_class_segment = pitchtools.PitchClassSegment(pitch_class_tokens)
    assert pytest.raises(AttributeError, "pitch_class_segment.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Chord___format___01():
    r'''Format chord with one note-head.
    '''

    chord = Chord("<cqs'>4")

    assert str(chord) == "<cqs'>4"
    assert format(chord) == "<cqs'>4"
    assert len(chord.note_heads) == 1
    assert len(chord.written_pitches) == 1


def test_scoretools_Chord___format___02():
    r'''Format chord with LilyPond command.
    '''

    chord = Chord("<d' ef' e'>4")
    command = indicatortools.LilyPondCommand('glissando', 'right')
    attach(command, chord)

    assert format(chord) == "<d' ef' e'>4 \\glissando"


def test_scoretools_Chord___format___03():
    r'''Format tweaked chord with LilyPond command.
    '''

    chord = Chord("<d' ef' e'>4")
    chord.note_heads[0].tweak.color = 'red'
    command = indicatortools.LilyPondCommand('glissando', 'right')
    attach(command, chord)

    assert format(chord) == stringtools.normalize(
        r'''
        <
            \tweak color #red
            d'
            ef'
            e'
        >4 \glissando
        '''
        )

    assert inspect_(chord).is_well_formed()


def test_scoretools_Chord___format___04():
    '''Format tweaked chord.
    '''

    chord = Chord("<d' ef' e'>4")
    chord.note_heads[0].tweak.transparent = True

    assert format(chord) == stringtools.normalize(
        r'''
        <
            \tweak transparent ##t
            d'
            ef'
            e'
        >4
        '''
        )


def test_scoretools_Chord___format___05():
    r'''Formats tweaked chord.
    '''

    chord = Chord("<d' ef' e'>4")
    chord.note_heads[0].tweak.style = 'harmonic'

    assert format(chord) == stringtools.normalize(
        r'''
        <
            \tweak style #'harmonic
            d'
            ef'
            e'
        >4
        '''
        )



# -*- coding: utf-8 -*-


def list_all_abjad_classes(modules=None, ignored_classes=None):
    r'''Lists all public classes defined in Abjad.

    ::

        >>> all_classes = documentationtools.list_all_abjad_classes()

    '''
    from abjad.tools import documentationtools
    return documentationtools.list_all_classes(
        modules='abjad',
        ignored_classes=ignored_classes,
        )



# -*- coding: utf-8 -*-
from abjad import *


def test_systemtools_LilyPondFormatManager_report_component_format_contributions_01():
    r'''You can report_component_format_contributions on a heavily
    tweaked leaf.
    '''

    t = Note("c'4")
    override(t).note_head.style = 'cross'
    override(t).note_head.color = 'red'
    override(t).stem.color = 'red'
    articulation = Articulation('staccato')
    attach(articulation, t)
    articulation = Articulation('tenuto')
    attach(articulation, t)
    markup = markuptools.Markup('some markup', Down)
    attach(markup, t)
    comment = indicatortools.LilyPondComment('textual information before', 'before')
    attach(comment, t)
    comment = indicatortools.LilyPondComment('textual information after', 'after')
    attach(comment, t)

    assert systemtools.LilyPondFormatManager.report_component_format_contributions(t) == \
        stringtools.normalize(
        r'''
        slot 1:
            comments:
                % textual information before
            grob overrides:
                \once \override NoteHead.color = #red
                \once \override NoteHead.style = #'cross
                \once \override Stem.color = #red
        slot 3:
        slot 4:
            leaf body:
                c'4 -\staccato -\tenuto _ \markup { "some markup" }
        slot 5:
        slot 7:
            comments:
                % textual information after
        ''')



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import systemtools


def test_scoretools_make_piano_score_from_leaves_01():
    r'''Works with notes.
    '''

    pitches = [-12, 37, -10, 27, 4, 17]
    notes = [Note(x, (1, 4)) for x in pitches]
    score, treble_staff, bass_staff = scoretools.make_piano_score_from_leaves(notes)

    r"""
    \new Score <<
        \new PianoStaff <<
            \context Staff = "treble" {
                \clef "treble"
                r4
                cs''''4
                r4
                ef'''4
                e'4
                f''4
            }
            \context Staff = "bass" {
                \clef "bass"
                c4
                r4
                d4
                r4
                r4
                r4
            }
        >>
    >>
    """

    assert inspect_(score).is_well_formed()
    assert format(score) == stringtools.normalize(
        r"""
        \new Score <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                    r4
                    cs''''4
                    r4
                    ef'''4
                    e'4
                    f''4
                }
                \context Staff = "bass" {
                    \clef "bass"
                    c4
                    r4
                    d4
                    r4
                    r4
                    r4
                }
            >>
        >>
        """
        )


def test_scoretools_make_piano_score_from_leaves_02():
    r'''Works with explicit lowest treble pitch.
    '''

    container = topleveltools.parse("{ g4 a4 b4 c'4 d'4 r4 a4 g4 }")
    container_contents = container[:]
    del(container[:])
    lowest_treble_pitch = NamedPitch('a')
    score, treble_staff, bass_staff = \
        scoretools.make_piano_score_from_leaves(
        container_contents,
        lowest_treble_pitch=lowest_treble_pitch,
        )

    r'''
    \new Score <<
        \new PianoStaff <<
            \context Staff = "treble" {
                \clef "treble"
                r4
                a4
                b4
                c'4
                d'4
                r4
                a4
                r4
            }
            \context Staff = "bass" {
                \clef "bass"
                g4
                r4
                r4
                r4
                r4
                r4
                r4
                g4
            }
        >>
    >>
    '''

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new PianoStaff <<
                \context Staff = "treble" {
                    \clef "treble"
                    r4
                    a4
                    b4
                    c'4
                    d'4
                    r4
                    a4
                    r4
                }
                \context Staff = "bass" {
                    \clef "bass"
                    g4
                    r4
                    r4
                    r4
                    r4
                    r4
                    r4
                    g4
                }
            >>
        >>
        '''
        )



# -*- coding: utf-8 -*-
import copy
from abjad import *


def test_datastructuretools_TypedOrderedDict___copy___01():

    dictionary_1 = datastructuretools.TypedOrderedDict([
        ('flavor', 'cherry'), ('count', 2),
        ])
    dictionary_2 = copy.copy(dictionary_1)

    assert dictionary_1 == dictionary_2
    assert repr(dictionary_1) == repr(dictionary_2)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Tuplet_context_settings_01():
    r'''Tuplet bracket context settings at before slot.
    '''

    tuplet = Tuplet((2, 3), "c'8 d'8 e'8 f'8")
    set_(tuplet).score.beam_exceptions = schemetools.SchemeVector()

    assert format(tuplet) == stringtools.normalize(
        r'''
        \set Score.beamExceptions = #'()
        \tweak edge-height #'(0.7 . 0)
        \times 2/3 {
            c'8
            d'8
            e'8
            f'8
        }
        '''
        )



# -*- coding: utf-8 -*-
import pytest
from abjad import *



@pytest.mark.skip('unskip me before building 2.15')
def test_topleveltools_play_01():
    r'''A note can be played.
    '''
    note = Note(1, (1, 2))
    play(note)


@pytest.mark.skip('unskip me before building 2.15')
def test_topleveltools_play_02():
    r'''A score can be played.
    '''
    notes = [Note(i, (1, 64)) for i in range(10)]
    score = Score([Staff(notes)])
    play(score)



# -*- coding: utf-8 -*-


def is_singleton(argument):
    r'''Is true when `argument` is a tuple of length 1. Otherwise false.

    ..  container:: example

        ::

            >>> mathtools.is_singleton((19,))
            True

        ::

            >>> mathtools.is_singleton((19, 20, 21))
            False

    Returns true or false.
    '''
    return isinstance(argument, tuple) and len(argument) == 1



# -*- coding: utf-8 -*-
from abjad import *
import copy


def test_scoretools_Rest___copy___01():
    r'''Copy rest.
    '''

    rest_1 = Rest((1, 4))
    rest_2 = copy.copy(rest_1)

    assert isinstance(rest_1, Rest)
    assert isinstance(rest_2, Rest)
    assert format(rest_1) == format(rest_2)
    assert rest_1 is not rest_2


def test_scoretools_Rest___copy___02():
    r'''Copy rest with LilyPond multiplier.
    '''

    rest_1 = Rest('r4')
    attach(Multiplier(1, 2), rest_1)
    rest_2 = copy.copy(rest_1)

    assert isinstance(rest_1, Rest)
    assert isinstance(rest_2, Rest)
    assert format(rest_1) == format(rest_2)
    assert rest_1 is not rest_2


def test_scoretools_Rest___copy___03():
    r'''Copy rest with LilyPond grob overrides and LilyPond context settings.
    '''

    rest_1 = Rest((1, 4))
    override(rest_1).staff.note_head.color = 'red'
    override(rest_1).accidental.color = 'red'
    set_(rest_1).tuplet_full_length = True
    rest_2 = copy.copy(rest_1)

    assert isinstance(rest_1, Rest)
    assert isinstance(rest_2, Rest)
    assert format(rest_1) == format(rest_2)
    assert rest_1 is not rest_2



# -*- coding: utf-8 -*-
import collections
import operator
from abjad.tools import mathtools
from abjad.tools.topleveltools.new import new
from abjad.tools.datastructuretools.TypedTuple import TypedTuple


class CompoundPattern(TypedTuple):
    r'''Compound pattern.

    ::
    
        >>> import abjad

    ..  container:: example

        Matches every index that is (one of the first three indices) OR (one of
        the last three indices):

        ::

            >>> pattern = patterntools.CompoundPattern(
            ...     [
            ...         patterntools.Pattern(
            ...             indices=[0, 1, 2],
            ...             ),
            ...         patterntools.Pattern(
            ...             indices=[-3, -2, -1],
            ...             ),
            ...         ],
            ...     )

        ::

            >>> print(format(pattern))
            patterntools.CompoundPattern(
                (
                    patterntools.Pattern(
                        indices=[0, 1, 2],
                        ),
                    patterntools.Pattern(
                        indices=[-3, -2, -1],
                        ),
                    ),
                operator='or',
                )

        ..  container:: example

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices):

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0],
                ...             period=2,
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             inverted=True,
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0],
                            period=2,
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            inverted=True,
                            ),
                        ),
                    operator='and',
                    )

        ..  container:: example

            Sieve from opening of Xenakis's Psappha:

            ::

                >>> sieve_1a = patterntools.select_every([0, 1, 7], period=8)
                >>> sieve_1b = patterntools.select_every([1, 3], period=5)
                >>> sieve_1 = sieve_1a & sieve_1b
                >>> sieve_2a = patterntools.select_every([0, 1, 2], period=8)
                >>> sieve_2b = patterntools.select_every([0], period=5)
                >>> sieve_2 = sieve_2a & sieve_2b
                >>> sieve_3 = patterntools.select_every([3], period=8)
                >>> sieve_4 = patterntools.select_every([4], period=8)
                >>> sieve_5a = patterntools.select_every([5, 6], period=8)
                >>> sieve_5b = patterntools.select_every([2, 3, 4], period=5)
                >>> sieve_5 = sieve_5a & sieve_5b
                >>> sieve_6a = patterntools.select_every([1], period=8)
                >>> sieve_6b = patterntools.select_every([2], period=5)
                >>> sieve_6 = sieve_6a & sieve_6b
                >>> sieve_7a = patterntools.select_every([6], period=8)
                >>> sieve_7b = patterntools.select_every([1], period=5)
                >>> sieve_7 = sieve_7a & sieve_7b
                >>> sieve = sieve_1 | sieve_2 | sieve_3 | sieve_4 | sieve_5 | sieve_6 | sieve_7

            ::

                >>> sieve.get_boolean_vector(total_length=40)
                [1, 1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 1, 0, 1, 1, 0, 1,
                1, 0, 1, 1, 0, 1, 0, 1, 1, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1, 0]

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_inverted',
        '_operator',
        )

    _name_to_operator = {
        'and': operator.and_,
        'or': operator.or_,
        'xor': operator.xor,
        }

    ### INITIALIZER ###

    def __init__(self, items=None, inverted=None, operator='or'):
        from abjad.tools import patterntools
        items = items or ()
        prototype = (patterntools.Pattern, type(self))
        for item in items:
            assert isinstance(item, prototype), repr(item)
        assert operator in self._name_to_operator, repr(operator)
        TypedTuple.__init__(
            self,
            items=items,
            )
        self._inverted = inverted
        self._operator = operator

    ### SPECIAL METHODS ###

    def __and__(self, pattern):
        r'''Logical AND of two patterns.

        ..  container:: example

            Flat grouping:

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern_3 = patterntools.select_every([0], period=2)
                >>> pattern = pattern_1 & pattern_2 & pattern_3

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            period=2,
                            ),
                        ),
                    operator='and',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]

        ..  container:: example

            Nested grouping:

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern_3 = patterntools.select_every([0], period=2)
                >>> pattern = pattern_1 & pattern_2 | pattern_3

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.CompoundPattern(
                            (
                                patterntools.Pattern(
                                    indices=[0, 1, 2],
                                    ),
                                patterntools.Pattern(
                                    indices=[-3, -2, -1],
                                    ),
                                ),
                            operator='and',
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            period=2,
                            ),
                        ),
                    operator='or',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0]

        Returns new compound pattern.
        '''
        if self._can_append_to_self(pattern, 'and'):
            patterns = self.items + [pattern]
            result = type(self)(patterns, operator='and')
        else:
            result = type(self)([self, pattern], operator='and')
        return result

    def __invert__(self):
        r'''Inverts pattern.

        ..  container:: example

            Matches every index that is (one of the first three indices) or
            (one of the last three indices):

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern = pattern_1 | pattern_2

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        ),
                    operator='or',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]

        ..  container:: example

            Matches every index that is NOT (one of the first three indices) or
            (one of the last three indices):

            ::

                >>> pattern = ~pattern
                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        ),
                    inverted=True,
                    operator='or',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0]

        Returns new compound pattern.
        '''
        inverted = not self.inverted
        pattern = new(self, inverted=inverted)
        return pattern

    def __or__(self, pattern):
        r'''Logical OR of two patterns.

        ..  container:: example

            Flat grouping:

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern_3 = patterntools.select_every([0], period=2)
                >>> pattern = pattern_1 | pattern_2 | pattern_3

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            period=2,
                            ),
                        ),
                    operator='or',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [1, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1, 1]

        ..  container:: example

            Nested grouping:

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern_3 = patterntools.select_every([0], period=2)
                >>> pattern = pattern_1 | pattern_2 & pattern_3

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.CompoundPattern(
                            (
                                patterntools.Pattern(
                                    indices=[-3, -2, -1],
                                    ),
                                patterntools.Pattern(
                                    indices=[0],
                                    period=2,
                                    ),
                                ),
                            operator='and',
                            ),
                        ),
                    operator='or',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0]

        Returns new compound pattern.
        '''
        if self._can_append_to_self(pattern, 'or'):
            patterns = self.items + [pattern]
            result = type(self)(patterns, operator='or')
        else:
            result = type(self)([self, pattern], operator='or')
        return result

    def __xor__(self, pattern):
        r'''Logical XOR of two patterns.

        ..  container:: example

            Flat grouping:

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern_3 = patterntools.select_every([0], period=2)
                >>> pattern = pattern_1 ^ pattern_2 ^ pattern_3

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            period=2,
                            ),
                        ),
                    operator='xor',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [0, 1, 0, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 0, 1]

        ..  container:: example

            Nested grouping:

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern_3 = patterntools.select_every([0], period=2)
                >>> pattern = pattern_1 ^ pattern_2 & pattern_3

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        patterntools.CompoundPattern(
                            (
                                patterntools.Pattern(
                                    indices=[-3, -2, -1],
                                    ),
                                patterntools.Pattern(
                                    indices=[0],
                                    period=2,
                                    ),
                                ),
                            operator='and',
                            ),
                        ),
                    operator='xor',
                    )

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0]

        Returns new compound pattern.
        '''
        if self._can_append_to_self(pattern, 'xor'):
            patterns = self.items + [pattern]
            result = type(self)(patterns, operator='xor')
        else:
            result = type(self)([self, pattern], operator='xor')
        return result

    ### PRIVATE METHODS ###

    def _can_append_to_self(self, pattern, operator_):
        from abjad.tools import patterntools
        if operator_ == self.operator:
            if isinstance(pattern, patterntools.Pattern):
                return True
            if (isinstance(pattern, type(self)) and
                pattern.operator == self.operator):
                return True
        return False

    ### PUBLIC METHODS ###

    def get_boolean_vector(self, total_length=None):
        r'''Gets boolean vector of pattern applied to input sequence with
        `total_length`.

        ..  container:: example

            Two-part pattern with logical OR:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             ),
                ...         ],
                ...     operator='or',
                ...     )

            ::

                >>> pattern.get_boolean_vector(4)
                [1, 1, 1, 1]

            ::

                >>> pattern.get_boolean_vector(8)
                [1, 1, 1, 0, 0, 1, 1, 1]


            ::

                >>> pattern.get_boolean_vector(16)
                [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]

            Matches every index that is (one of the first three indices) OR
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with mixed periodic and inverted parts:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0],
                ...             period=2,
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             inverted=True,
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )

            ::

                >>> pattern.get_boolean_vector(4)
                [1, 0, 0, 0]

            ::

                >>> pattern.get_boolean_vector(8)
                [1, 0, 1, 0, 1, 0, 0, 0]

            ::

                >>> pattern.get_boolean_vector(16)
                [1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 1, 0, 0, 0]

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices).

        ..  container:: example

            Cyclic pattern that selects every fourth and fifth item:

            ::

                >>> pattern_1 = patterntools.Pattern([0], period=4)
                >>> pattern_2 = patterntools.Pattern([0], period=5)
                >>> pattern = pattern_1 | pattern_2

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0],
                            period=4,
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            period=5,
                            ),
                        ),
                    operator='or',
                    )

            ::

                >>> pattern.get_boolean_vector(4)
                [1, 0, 0, 0]

            ::

                >>> pattern.get_boolean_vector(8)
                [1, 0, 0, 0, 1, 1, 0, 0]

            ::

                >>> pattern.get_boolean_vector(16)
                [1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1]

            Sets total length to period of pattern when `total_length` is none:

            ::

                >>> pattern.period
                20

            ::

                >>> pattern.get_boolean_vector()
                [1, 0, 0, 0, 1, 1, 0, 0, 1, 0, 1, 0, 1, 0, 0, 1, 1, 0, 0, 0]

            ::

                >>> pattern.period == len(pattern.get_boolean_vector())
                True

        Returns list of ones and zeroes.
        '''
        total_length = total_length or self.period
        boolean_vector = []
        for index in range(total_length):
            result = self.matches_index(index, total_length)
            boolean_vector.append(int(result))
        return boolean_vector

    def get_matching_items(self, sequence):
        r'''Gets maching items from sequence.

        ..  container:: example

            ::

                >>> pattern = abjad.select_first(1) | abjad.select_last(2)

            ::

                >>> pattern.get_matching_items('abcdefghijklmnopqrstuvwxyz')
                Sequence(['a', 'y', 'z'])

        Returns new sequence.
        '''
        import abjad
        assert isinstance(sequence, collections.Iterable), repr(sequence)
        length = len(sequence)
        items = []
        for i in range(length):
            if self.matches_index(i, length):
                item = sequence[i]
                items.append(item)
        return abjad.Sequence(items=items)

    def matches_index(self, index, total_length, rotation=None):
        r'''Is true when compound pattern matches `index` under
        `total_length`. Otherwise false.

        ..  container:: example

            Empty pattern:

            ::

                >>> pattern = patterntools.CompoundPattern()

            Total length 16:

            ::

                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3
                4
                5
                6
                7
                8
                9
                10
                11
                12
                13
                14
                15

            Total length 8:

            ::

                >>> total_length = 8
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3
                4
                5
                6
                7

            Total length 4:

            ::

                >>> total_length = 4
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3

            Matches nothing.

        ..  container:: example

            Simple pattern:

            Logical OR:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         ],
                ...     operator='or',
                ...     )
                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4
                5
                6
                7
                8
                9
                10
                11
                12
                13
                14
                15

            Logical AND:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )
                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4
                5
                6
                7
                8
                9
                10
                11
                12
                13
                14
                15

            Logical XOR:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         ],
                ...     operator='xor',
                ...     )
                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4
                5
                6
                7
                8
                9
                10
                11
                12
                13
                14
                15

            Matches every index that is (one of the first three indices).

            Ignores `operator`.

        ..  container:: example

            Two-part pattern with logical OR:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             ),
                ...         ],
                ...     operator='or',
                ...     )

            Total length 16:

            ::

                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4
                5
                6
                7
                8
                9
                10
                11
                12
                13 True
                14 True
                15 True

            Total length 8:

            ::

                >>> total_length = 8
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4
                5 True
                6 True
                7 True

            Total length 4:

            ::

                >>> total_length = 4
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3 True

            Matches every index that is (one of the first three indices) OR
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with logical AND:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )

            Total length 16:

            ::

                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3
                4
                5
                6
                7
                8
                9
                10
                11
                12
                13
                14
                15

            Total length 8:

            ::

                >>> total_length = 8
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1
                2
                3
                4
                5
                6
                7

            Total length 4:

            ::

                >>> total_length = 4
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0
                1 True
                2 True
                3

            Matches every index that is (one of the first three indices) AND
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with logical XOR:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             ),
                ...         ],
                ...     operator='xor',
                ...     )

            Total length 16:

            ::

                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4
                5
                6
                7
                8
                9
                10
                11
                12
                13 True
                14 True
                15 True

            Total length 8:

            ::

                >>> total_length = 8
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4
                5 True
                6 True
                7 True

            Total length 4:

            ::

                >>> total_length = 4
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2
                3 True

            Matches every index that is (one of the first three indices) XOR
            (one of the last three indices).

        ..  container:: example

            Two-part pattern with mixed periodic and inverted parts:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0],
                ...             period=2,
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             inverted=True,
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )

            Total length 16:

            ::

                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2 True
                3
                4 True
                5
                6 True
                7
                8 True
                9
                10 True
                11
                12 True
                13
                14
                15

            Total length 8:

            ::

                >>> total_length = 8
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2 True
                3
                4 True
                5
                6
                7

            Total length 4:

            ::

                >>> total_length = 4
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1
                2
                3

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices).

        ..  container:: example

            Complex pattern with compound and simple parts:

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0],
                ...             period=2,
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             inverted=True,
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )
                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         pattern,
                ...         patterntools.Pattern(
                ...             indices=[0, 1, 2],
                ...             ),
                ...         ],
                ...     operator='or',
                ...     )

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.CompoundPattern(
                            (
                                patterntools.Pattern(
                                    indices=[0],
                                    period=2,
                                    ),
                                patterntools.Pattern(
                                    indices=[-3, -2, -1],
                                    inverted=True,
                                    ),
                                ),
                            operator='and',
                            ),
                        patterntools.Pattern(
                            indices=[0, 1, 2],
                            ),
                        ),
                    operator='or',
                    )

            Total length 16:

            ::

                >>> total_length = 16
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4 True
                5
                6 True
                7
                8 True
                9
                10 True
                11 
                12 True
                13
                14
                15

            Total length 8:

            ::

                >>> total_length = 8
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3
                4 True
                5
                6
                7

            Total length 4:

            ::

                >>> total_length = 4
                >>> for index in range(total_length):
                ...     match = pattern.matches_index(index, total_length)
                ...     match = match or ''
                ...     print(index, match)
                0 True
                1 True
                2 True
                3

            Matches every index that is ((equal to 0 % 2) AND (not one of the
            last three indices)) OR is (one of the first three indices).

        Returns true or false.
        '''
        if not self.items:
            result = False
        elif len(self.items) == 1:
            pattern = self.items[0]
            result = pattern.matches_index(
                index,
                total_length,
                rotation=rotation,
                )
        else:
            operator_ = self._name_to_operator[self.operator]
            pattern = self.items[0]
            result = pattern.matches_index(
                index,
                total_length,
                rotation=rotation,
                )
            for pattern in self.items[1:]:
                result_ = pattern.matches_index(
                    index,
                    total_length,
                    rotation=rotation,
                    )
                result = operator_(result, result_)
        if self.inverted:
            result = not(result)
        return result

    def reverse(self):
        r'''Reverses compound pattern.

        ..  container:: example

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices):

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0],
                ...             period=2,
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             inverted=True,
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0],
                            period=2,
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            inverted=True,
                            ),
                        ),
                    operator='and',
                    )

            Reverses pattern:

            ::

                >>> pattern = pattern.reverse()
                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[-1],
                            period=2,
                            ),
                        patterntools.Pattern(
                            indices=[2, 1, 0],
                            inverted=True,
                            ),
                        ),
                    operator='and',
                    )

            New pattern matches every index that is (equal to -1 % 2) AND
            (not one of the first three indices).

        Returns new compound pattern.
        '''
        patterns = [_.reverse() for _ in self]
        return new(self, items=patterns)

    def rotate(self, n=0):
        r'''Rotates compound pattern by index `n`.

        ..  container:: example

            Matches every index that is (equal to 0 % 2) AND (not one of the
            last three indices):

            ::

                >>> pattern = patterntools.CompoundPattern(
                ...     [
                ...         patterntools.Pattern(
                ...             indices=[0],
                ...             period=2,
                ...             ),
                ...         patterntools.Pattern(
                ...             indices=[-3, -2, -1],
                ...             inverted=True,
                ...             ),
                ...         ],
                ...     operator='and',
                ...     )

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0],
                            period=2,
                            ),
                        patterntools.Pattern(
                            indices=[-3, -2, -1],
                            inverted=True,
                            ),
                        ),
                    operator='and',
                    )

            Rotates pattern two elements to the right:

            ::

                >>> pattern = pattern.rotate(n=2)
                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[2],
                            period=2,
                            ),
                        patterntools.Pattern(
                            indices=[-1, 0, 1],
                            inverted=True,
                            ),
                        ),
                    operator='and',
                    )

            New pattern matches every index that is (equal to 2 % 2) AND (not
            the first, second or last index in the pattern).

        Returns new compound pattern.
        '''
        patterns = [_.rotate(n=n) for _ in self]
        return new(self, items=patterns)

    ### PUBLIC PROPERTIES ###

    @property
    def inverted(self):
        '''Is true when compound pattern is inverted. Otherwise false.

        ..  container:: example

            Matches every index that is (one of the first three indices) OR
            (one of the last three indices):

            ::

                >>> pattern_1 = patterntools.select_first(3)
                >>> pattern_2 = patterntools.select_last(3)
                >>> pattern = pattern_1 | pattern_2
                >>> pattern.inverted is None
                True

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1]


        ..  container:: example

            Matches every index that is NOT (one of the first three indices) OR
            (one of the last three indices):

            ::

                >>> pattern = new(pattern, inverted=True)
                >>> pattern.inverted
                True

            ::

                >>> pattern.get_boolean_vector(total_length=16)
                [0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0]

        Defaults to none.

        Set to true, false or none.

        Returns true, false or none.
        '''
        return self._inverted

    @property
    def operator(self):
        r'''Gets operator of compounnd pattern.

        Set to string.

        Returns string.
        '''
        return self._operator

    @property
    def period(self):
        r'''Gets period of compound pattern.

        ..  container:: example

            Gets period of pattern that selects every fourth and fifth element:

            ::

                >>> pattern_1 = patterntools.Pattern([0], period=4)
                >>> pattern_2 = patterntools.Pattern([0], period=5)
                >>> pattern = pattern_1 | pattern_2

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0],
                            period=4,
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            period=5,
                            ),
                        ),
                    operator='or',
                    )

            ::

                >>> pattern.period
                20

        ..  container:: example

            Returns none when pattern contains acyclic parts:

            ::

                >>> pattern_1 = patterntools.Pattern([0], period=4)
                >>> pattern_2 = patterntools.Pattern([0])
                >>> pattern = pattern_1 | pattern_2

            ::

                >>> print(format(pattern))
                patterntools.CompoundPattern(
                    (
                        patterntools.Pattern(
                            indices=[0],
                            period=4,
                            ),
                        patterntools.Pattern(
                            indices=[0],
                            ),
                        ),
                    operator='or',
                    )

            ::

                >>> pattern.period is None
                True

        Returns positive integer.
        '''
        periods = [_.period for _ in self]
        if None not in periods:
            return mathtools.least_common_multiple(*periods)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_Score_add_final_bar_line_01():

    staff = Staff("c'4 d'4 e'4 f'4")
    score = Score([staff])
    score.add_final_bar_line()

    r'''
    \new Score <<
        \new Staff {
            c'4
            d'4
            e'4
            f'4
            \bar "|."
        }
    >>
    '''

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                c'4
                d'4
                e'4
                f'4
                \bar "|."
            }
        >>
        '''
        )



# -*- coding: utf-8 -*-
from abjad.tools.datastructuretools import TreeContainer


class GraphvizGroup(TreeContainer):
    r'''A Graphviz struct field grouping.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = ()

    ### INITIALIZER ###

    def __init__(
        self,
        children=None,
        name=None,
        ):
        TreeContainer.__init__(
            self,
            children=children,
            name=name,
            )

    ### PRIVATE PROPERTIES ###

    @property
    def _node_class(self):
        from abjad.tools import graphtools
        prototype = (
            graphtools.GraphvizField,
            graphtools.GraphvizGroup,
            )
        return prototype

    @property
    def _struct_format_contributions(self):
        if not self:
            return ''
        result = []
        for x in self:
            part = x._struct_format_contributions
            if part:
                result.append(part)
        result = ' | '.join(result)
        result = '{{ {} }}'.format(result)
        return result



# -*- coding: utf-8 -*-
from abjad import *
import pytest


def test_scoretools_FixedDurationContainer_format_01():

    container = scoretools.FixedDurationContainer((3, 8), "c'8 d'8 e'8")
    assert format(container) == stringtools.normalize(
        r'''
        {
            c'8
            d'8
            e'8
        }
        '''
        )


def test_scoretools_FixedDurationContainer_format_02():

    container = scoretools.FixedDurationContainer((3, 8), "c'8 d'8")
    assert pytest.raises(Exception, 'container.format')


def test_scoretools_FixedDurationContainer_format_03():

    container = scoretools.FixedDurationContainer((3, 8), "c'8 d'8 e'8 f'8")
    assert pytest.raises(Exception, 'container.format')



# -*- coding: utf-8 -*-
import os
from abjad import *
configuration = systemtools.AbjadConfiguration()
ly_path = os.path.join(
    configuration.abjad_directory, 
    'test.ly',
    )


def test_agenttools_PersistenceAgent_as_ly_01():
    r'''Agent persists LilyPond file when no LilyPond file exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=[ly_path]):
        result = persist(note).as_ly(ly_path)
        assert os.path.isfile(ly_path)
        assert isinstance(result, tuple)

        
def test_agenttools_PersistenceAgent_as_ly_02():
    r'''Agent persists LilyPond file when LilyPond file already exists.
    '''

    note = Note("c'4")
    with systemtools.FilesystemState(remove=[ly_path]):
        result = persist(note).as_ly(ly_path)
        assert isinstance(result, tuple)
        assert os.path.isfile(ly_path)
        persist(note).as_ly(ly_path)
        assert os.path.isfile(ly_path)
